@@ -9,7 +9,7 @@ use strict;
 use warnings;
 use base qw( IO::Async::Timer );
 
-our $VERSION = '0.57';
+our $VERSION = '0.58';
 
 use Carp;
 
@@ -146,7 +146,7 @@ sub _make_cb
    my $self = shift;
 
    return $self->_capture_weakself( sub {
-      my ( $self ) = @_;
+      my $self = shift or return;
 
       undef $self->{id};
       $self->{expired} = 1;
